"""
Functions to display an error (error, warning or information) message.
"""

import hachoir.config as config
from hachoir.log import log
from hachoir.tools import getBacktrace

class HachoirError(Exception):
    """
    Parent of all errors in Hachoir library
    """
    pass

def info(message):
    """
    Display information message. Skip it in config.quiet is set.
    """
    if config.quiet:
        return
    if config.verbose:
        log.info(message)

def warning(message):
    """
    Display warning message. Skip it in config.quiet is set.
    """
    if config.quiet:
        return
    if config.debug:
        message += "\n\n" + getBacktrace()
    log.warning(message)

def error(message):
    """
    Display error message. If config.quiet or config.set is set, add
    backtrace to the message
    """
    if config.verbose or config.debug:
        message += "\n\n" + getBacktrace()
    log.error(message)

