"""
Bit sized classes:
- Bit: Single bit, value is False or True ;
- Bits: Integer with a size in bits ;
- RawBits: unknow content with a size in bits.
"""

from hachoir.field import Field
from hachoir.bits import countBits

class RawBits(Field):
    """
    Unknow content with a size in bits.
    """
    static_size = staticmethod(lambda *args, **kw: args[1])

    def __init__(self, parent, name, size, description=None):
        """
        Constructor: see L{Field.__init__} for parameter description
        """
        assert issubclass(parent.__class__, Field)
        # Arbitrary limit (stream supports bigger value)
        assert size <= 256
        Field.__init__(self, parent, name, size=size, description=description)

    def _createValue(self):
        return self._parent.stream.readBits(
            self.absolute_address, self._size, self._parent.endian)

class Bits(RawBits):
    """
    Positive integer with a size in bits

    @see: L{Bit}
    @see: L{RawBits}
    """
    pass

class Bit(RawBits):
    """
    Single bit: value can be False or True, and size is exactly one bit.

    @see: L{Bits}
    """
    static_size = 1

    def __init__(self, parent, name, description=None):
        """
        Constructor: see L{Field.__init__} for parameter description
        """
        RawBits.__init__(self, parent, name, 1, description=description)

    def _createValue(self):
        return 1 == self._parent.stream.readBits(
                self.absolute_address, 1, self._parent.endian)

