"""
Very basic field: raw content with a size in byte. Use this class for
unknow content.
"""

from hachoir.field import Field, FieldError
from hachoir.tools import makePrintable
from hachoir import config

class RawBytes(Field):
    """
    Byte vector of unknow content

    @see: L{Bytes}
    """
    static_size = staticmethod(lambda *args, **kw: args[1]*8)

    def __init__(self, parent, name, length, description="Raw data"):
        assert issubclass(parent.__class__, Field)
        if not(0 < length):
            raise FieldError("Invalid RawBytes length (%s)!" % length)
        assert length < (1 << 64) # arbitrary limit
        Field.__init__(self, parent, name, length*8, description)
        self._display = None

    def _getDisplay(self):
        max_bytes = config.max_byte_length
        if self._value is None:
            if self._display is None:
                address = self.absolute_address
                length = min(self._size / 8, max_bytes)
                self._display = self._parent.stream.readBytes(address, length)
            display = self._display
        else:
            display = self._value[:max_bytes]
        if 8 * len(display) < self._size:
            display += "(...)"
        return makePrintable(display, "latin-1", quote=True)

    def _createValue(self):
        assert (self._size % 8) == 0
        if self._display:
            self._display = None
        return self._parent.stream.readBytes(
            self.absolute_address, self._size / 8)

class Bytes(RawBytes):
    """
    Byte vector: can be used for magic number or GUID/UUID for example.

    @see: L{RawBytes}
    """
    pass

