def Enum(field, enum, key_func=None):
    """
    Enum is an adapter to another field: it will just change its display
    attribute. It uses a dictionnary to associate a value to another.

    key_func is an optional function with prototype "def func(key)->key"
    which is called to transform key.
    """
    display = field._getDisplay
    field.getOriginalDisplay = display
    if key_func:
        def _getDisplay():
            try:
                key = key_func(field.value)
                return enum[key]
            except (IndexError, KeyError):
                return display()
    else:
        def _getDisplay():
            try:
                return enum[field.value]
            except (IndexError, KeyError):
                return display()
    field._getDisplay = _getDisplay
    field.getEnum = lambda: enum
    return field

