from hachoir.field import Bits
from hachoir.endian import BIG_ENDIAN, LITTLE_ENDIAN
import struct

class BasicFloat(Bits):
    def __init__(self, parent, name, format, size, description=None):
        assert (size % 8) == 0
        assert parent.endian in (BIG_ENDIAN, LITTLE_ENDIAN)
        Bits.__init__(self, parent, name, size, description)
        self.format = format
        if self._parent.endian == BIG_ENDIAN:
            self.real_format = ">"+format
        else:
            self.real_format = "<"+format

    def _createValue(self):
        raw = self._parent.stream.readBytes(
            self.absolute_address, self._size/8)
        raw = struct.unpack(self.real_format, raw)
        assert len(raw) == 1
        return raw[0]

def BasicFloatFactory(format, size):
    class Float(BasicFloat):
        static_size = size
        def __init__(self, parent, name, description=None):
            BasicFloat.__init__(self, parent, name, format, size, description)
    return Float

# 32 bits float (standart: IEEE 754)
Float32 = BasicFloatFactory("f", 32)

Float64 = BasicFloatFactory("d", 64)

