"""
Integer field classes:
- UInt8, UInt16, UInt32, UInt64: unsigned integer of 8, 16, 32, 64 bits ;
- Int8, Int16, Int32, Int64: signed integer of 8, 16, 32, 64 bits.

Constructor has an optional argument: text_handler, which is a function
called to create display attribute. You can use for example
hexadecimal() from hachoir.text_handler module.
"""

import types
from hachoir.field import Bits

class GenericInteger(Bits):
    """
    Generic class used to generate other classes:
    - unsigned integer: UInt8, UInt16, UInt32, UInt64 ;
    - signed integer: Int8, Int16, Int32, Int64.
    """
    def __init__(self, parent, name, signed, size, description=None,
    text_handler=None):
        Bits.__init__(self, parent, name, size, description)
        self.signed = signed
        if text_handler:
            assert isinstance(text_handler, types.FunctionType) \
                or isinstance(text_handler, types.MethodType)
            self._getDisplay = lambda: text_handler(self)

    def _createValue(self):
        return self._parent.stream.readInteger(
            self.absolute_address, self.signed, self._size, self._parent.endian)

def IntegerFactory(signed, size):
    class Integer(GenericInteger):
        static_size = size
        def __init__(self, parent, name, description=None, text_handler=None):
            GenericInteger.__init__(self, parent, name, signed, size, description, text_handler)
    return Integer

UInt8 = IntegerFactory(False, 8)
UInt16 = IntegerFactory(False, 16)
UInt32 = IntegerFactory(False, 32)
UInt64 = IntegerFactory(False, 64)

Int8 = IntegerFactory(True, 8)
Int16 = IntegerFactory(True, 16)
Int32 = IntegerFactory(True, 32)
Int64 = IntegerFactory(True, 64)

