from hachoir.field import Bits, Bytes
from hachoir.error import info, warning
from hachoir import config

class PaddingBits(Bits):
    static_size = staticmethod(lambda *args, **kw: args[1])

    def __init__(self, parent, name, nbits, description="Padding", pattern=None):
        Bits.__init__(self, parent, name, nbits, description)
        self.pattern = pattern
        # TODO: Check pattern

class PaddingBytes(Bytes):
    static_size = staticmethod(lambda *args, **kw: args[1]*8)

    def __init__(self, parent, name, nbytes,
    description="Padding", pattern=None):
        """ pattern is None or repeated string """
        assert (pattern is None) or (isinstance(pattern, str))
        Bytes.__init__(self, parent, name, nbytes, description)
        self.pattern = pattern
        if config.check_padding_pattern:
            self.checkPattern()

    def checkPattern(self):
        if self.pattern is None:
            return

        length = self._size/8
        if 4096 < length:
            info("Notice: only check first 4KB of padding")
            length = 4096
        content = self._parent.stream.readBytes(self.absolute_address, length)
        index = 0
        pattern_len = len(self.pattern)
        while index < length:
            if content[index:index+pattern_len] != self.pattern:
                warning(
                    "Warning: padding %s content don't look normal"
                    " (invalid pattern at byte %u)!"
                    % (self.path, index))
                return
            index += pattern_len

