"""
BZIP2 archive file

Author: Victor Stinner
"""

from hachoir.field import (Parser, ParserError,
    RawBytes, String, Character, UInt8, UInt32)
from hachoir.endian import LITTLE_ENDIAN
from hachoir.text_handler import hexadecimal

class Bzip2File(Parser):
    tags = {
        "file_ext": "bz2",
        "mime": ["application/x-bzip2"],
        "min_size": 10*8,
        "description": "bzip2 archive"
    }
    endian = LITTLE_ENDIAN

    def validate(self):
        if self.stream.readBytes(0, 3) != 'BZh':
            return "Wrong file signature"
#        if not("1" <= self["blocksize"].value <= "9"):
#                "Wrong blocksize"
        return True

    def createFields(self):
        yield String(self, "id", 3, "Identifier (BZh)", charset="ascii")
        yield Character(self, "blocksize", "Block size (KB of memory needed to uncompress)")

        yield UInt8(self, "blockheader", "Block header")
        if self["blockheader"].value == 0x17:
            yield String(self, "id2", 4, "Identifier2 (re8P)", charset="ascii")
            yield UInt8(self, "id3", "Identifier3 (0x90)")
        elif self["blockheader"].value == 0x31:
            yield String(self, "id2", 5, "Identifier 2 (AY&SY)", charset="ascii")
            if self["id2"].value != "AY&SY":
                raise ParserError("Invalid identifier 2 (AY&SY)!")
        else:
            raise ParserError("Invalid block header!")
        yield UInt32(self, "crc32", "CRC32", text_handler=hexadecimal)

        size = (self.size - self.current_size)/8
        yield RawBytes(self, "content", size, "Compressed data content")

