from hachoir.field import FieldSet, UInt8

class RGB(FieldSet):
    color_name = {
        (  0,   0,   0): "Black",
        (255,   0,   0): "Red",
        (  0, 255,   0): "Green",
        (  0,   0, 255): "Blue",
        (255, 255, 255): "White",
    }
    static_size = 3*8

    def createFields(self):
        yield UInt8(self, "red", "Red")
        yield UInt8(self, "green", "Green")
        yield UInt8(self, "blue", "Blue")

    def _createDescription(self):
        rgb = self["red"].value, self["green"].value, self["blue"].value
        name = self.color_name.get(rgb)
        if not name:
            name = "#%02X%02X%02X" % rgb
        return "RGB color: " + name

class RGBA(RGB):
    static_size = 4*8
    def createFields(self):
        yield UInt8(self, "red", "Red")
        yield UInt8(self, "green", "Green")
        yield UInt8(self, "blue", "Blue")
        yield UInt8(self, "alpha", "Alpha")

    def _createDescription(self):
        description = RGB._createDescription(self)
        opacity = self["alpha"].value*100/255
        return "%s (opacity: %s%%)" % (description, opacity)

class Palette(FieldSet):
    def __init__(self, parent, name, nb_colors, description=None):
        assert 0 < nb_colors
        self.nb_colors = nb_colors
        size = self.nb_colors * RGB.static_size
        FieldSet.__init__(self, parent, name, description, size=size)

    def _createDescription(self):
        return "Palette of %u RGB colors" % self.nb_colors

    def createFields(self):
        for index in xrange(self.nb_colors):
            yield RGB(self, "color[]")

class PaletteRGBA(FieldSet):
    def __init__(self, parent, name, nb_colors, description=None):
        assert 0 < nb_colors
        self.nb_colors = nb_colors
        size = self.nb_colors * RGBA.static_size
        FieldSet.__init__(self, parent, name, description, size=size)

    def _createDescription(self):
        return "Palette of %u RGBA colors" % self.nb_colors

    def createFields(self):
        for i in xrange(self.nb_colors):
            yield RGBA(self, "color[]")

