#
# Ogg parser
# Author Julien Muchembled <jm AT jm10.no-ip.com>
# Created: 10 june 2006
#

from hachoir.field import (Field, FieldSet, Parser,
    RawBits, Bit, UInt8, UInt32, UInt64,
    RawBytes, String)
from hachoir.endian import LITTLE_ENDIAN
from hachoir.error import warning

class XiphInt(Field):
    def __init__(self, parent, name, max_size=None, description=None):
        Field.__init__(self, parent, name, size=0, description=description)
        value = 0
        while max_size is None or self._size < max_size:
            byte = parent.stream.readBits(self.absolute_address + self._size, 8, LITTLE_ENDIAN)
            value += byte
            self._size += 8
            if byte != 0xff:
                break
        self._createValue = lambda: value

class Lacing(FieldSet):
    def createFields(self):
        while self.current_size < self._size:
            yield XiphInt(self, 'size[]', self._size - self.current_size)

class OggPage(FieldSet):
    def __init__(self, *args):
        FieldSet.__init__(self, *args)
        size = 27
        self.lacing_size = self['lacing_size'].value
        if self.lacing_size:
            size += self.lacing_size
            lacing = self['lacing']
            self.packet_size = [ field.value for field in lacing ]
            size += sum(self.packet_size)
        self._size = size * 8

    def createFields(self):
        yield String(self, 'capture_pattern', 4, charset="ascii")
        if self['capture_pattern'].value != 'OggS':
            warning('Invalid signature. An Ogg page must start with "OggS".')
        yield UInt8(self, 'stream_structure_version')
        yield Bit(self, 'continued_packet')
        yield Bit(self, 'first_page')
        yield Bit(self, 'last_page')
        yield RawBits(self, 'unused', 5)
        yield UInt64(self, 'abs_granule_pos')
        yield UInt32(self, 'serial')
        yield UInt32(self, 'page')
        yield UInt32(self, 'checksum')
        yield UInt8(self, 'lacing_size')
        if self.lacing_size:
            yield Lacing(self, "lacing", size=self.lacing_size*8)
            index = 0
            for packet_size in self.packet_size:
                if packet_size:
                    yield RawBytes(self, 'packet[%u]' % index, packet_size)
                index += 1

class OggFile(Parser):
    tags = {
        "file_ext": ("ogg", "ogm"),
        "mime": ["application/ogg"],
        "min_size": 28*8,
        "description": "Ogg multimedia container"
    }
    endian = LITTLE_ENDIAN

    def validate(self):
        return (self.stream.readBytes(0, 4) == "OggS")

    def createFields(self):
        while self.current_size < self.size:
            yield OggPage(self, "page[]")

