"""
Microsoft Windows Portable Executable (PE) file parser.

Author: Victor Stinner
Creation date: 2006-08-13
"""

# TODO: Just keep what you need
from hachoir.field import (Parser, ParserError, FieldSet,
    UInt16, UInt32,
    String,
    PaddingBytes, RawBytes)
from hachoir.endian import LITTLE_ENDIAN, BIG_ENDIAN
from hachoir.field.static_field_set import StaticFieldSet

class MSDosHeader(StaticFieldSet):
    format = (
        (String, "header", 2, "File header (MZ)", {"charset": "ascii"}),
        (UInt16, "filesize_mod_512", "File size in bytes modulo 512"),
        (UInt16, "filesize_div_512", "File size in bytes divide by 512"),
        (UInt16, "reloc_entries", "Number of relocation entries"),
        (UInt16, "code_offset", "Offset to the code in the file (divided by 16)"),
        (UInt16, "needed_memory", "Memory needed to run (divided by 16)"),
        (UInt16, "max_memory", "Maximum memory needed to run (divided by 16)"),
        (UInt32, "init_ss_sp", "Initial value of SP:SS registers."),
        (UInt16, "checksum", "Checksum"),
        (UInt32, "init_cs_ip", "Initial value of CS:IP registers."),
        (UInt16, "reloc_offset", "Offset in file to relocation table"),
        (UInt16, "overlay_number", "Overlay number"),
        (String, "reserved[]", 8, "Reserverd"),
        (UInt16, "oem_id", "OEM id"),
        (UInt16, "oem_info", "OEM info"),
        (PaddingBytes, "reserved[]", 20, "Reserved"),
        (UInt32, "pe_offset", "Offset to PE header"))

class ExeFile(Parser):
    tags = {
        "file_ext": "exe",
        "mime": ("application/x-dosexec",),
        "min_size": 64*8,
        "description": "Microsoft Windows Portable Executable"
    }
    endian = LITTLE_ENDIAN

    def validate(self):
        if self.stream.readBytes(0, 2) != 'MZ':
            return "Wrong header"
        return True

    def createFields(self):
        yield MSDosHeader(self, "msdos_header", "MS-Dos program header")

        # Read rest of the file (if any)
        size = (self.size - self.current_size) / 8
        if size:
            yield RawBytes(self, "end", size)

