from cStringIO import StringIO
from hachoir.bits import countBits
from hachoir.endian import BIG_ENDIAN, LITTLE_ENDIAN

class OutputStream(object):
    def __init__(self, output, endian=BIG_ENDIAN, filename=None):
        self._output = output
        self._filename = filename
        self._bit_pos = 0
        self._byte = 0
        self._endian = endian

    def _getFilename(self):
        return self._filename
    filename = property(_getFilename)

    def writeBit(self, state):
        if self._bit_pos == 7:
            self._bit_pos = 0
            if state:
                if self._endian is BIG_ENDIAN:
                    self._byte |= 1
                else:
                    self._byte |= 128
            self._output.write(chr(self._byte))
            self._byte = 0
        else:
            if state:
                if not self._endian is BIG_ENDIAN:
                    self._byte |= (1 << (7-self._bit_pos))
                else:
                    self._byte |= (1 << self._bit_pos)
            self._bit_pos += 1

    def writeBits(self, count, value, endian):
        # --- TODO: Fix that :-)
        endian = BIG_ENDIAN
        # --- end of TODO

        assert countBits(value) <= count

        # Feed bits to align to byte address
        if self._bit_pos != 0:
            n = 8 - self._bit_pos
            if n <= count:
                count -= n
                if endian is BIG_ENDIAN:
                    self._byte |= (value >> count)
                    value &= ((1 << count) - 1)
                else:
                    self._byte |= (value & ((1 << n)-1)) << self._bit_pos
                    value >>= n
                self._output.write(chr(self._byte))
                self._byte = 0
            else:
                if endian is not BIG_ENDIAN:
                    self._byte |= (value << (7-self._bit_pos-count))
                else:
                    self._byte |= (value << self._bit_pos)
                self._bit_pos += count
                return

        # Write byte per byte
        while 8 <= count:
            count -= 8
            if endian is BIG_ENDIAN:
                byte = (value >> count)
                value &= ((1 << count) - 1)
            else:
                byte = (value & 0xFF)
                value >>= 8
            self._output.write(chr(byte))

        # Keep last bits
        assert 0 <= count and count < 8
        self._bit_pos = count
        if 0 < count:
            assert countBits(value) <= count
            if endian is not BIG_ENDIAN:
                self._byte = value << (8-count)
            else:
                self._byte = value
        else:
            assert value == 0
            self._byte = 0

    def copyBitsFrom(self, input, address, nb_bits, endian):
        if (nb_bits % 8) == 0 and endian is BIG_ENDIAN:
            self.copyBytesFrom(input, address, nb_bits/8)
        else:
            # Arbitrary limit (because we should use a buffer, like copyBytesFrom(),
            # but with endianess problem
            assert nb_bits <= 128
            data = input.readBits(address, nb_bits, endian)
            self.writeBits(nb_bits, data, True)

    def copyBytesFrom(self, input, address, nb_bytes):
        buffer_size = 1 << 16 # 64 KB
        while 0 < nb_bytes:
            # Compute buffer size
            if nb_bytes < buffer_size:
                buffer_size = nb_bytes

            # Read
            data = input.readBytes(address, buffer_size)

            # Write
            self.writeBytes(data)

            # Move address
            address += buffer_size*8
            nb_bytes -= buffer_size

    def writeBytes(self, bytes):
        if self._bit_pos != 0:
            raise NotImplementedError()
        self._output.write(bytes)

    def paddingToByte(self, bit=False):
        n = 0
        while self._bit_pos != 0:
            self.writeBit(bit)
            n += 1
        return n

class StringOutputStream(OutputStream):
    """ Write data into a string """
    def __init__(self, endian=BIG_ENDIAN):
        self.string = StringIO()
        OutputStream.__init__(self, self.string, \
            endian=endian, filename=None)

def FileOutputStream(filename, endian=BIG_ENDIAN):
    """ Write data into a file """
    output = open(filename, 'w')
    return OutputStream(output, endian=endian, filename=filename)

