from hachoir.tools import humanFilesize
import errno
import sys

class TextOutput:
    def __init__(self, field_set, charset, max_depth, options):
        self.field_set = field_set
        self.charset = charset
        self.max_depth = max_depth
        self.display_value = options.get("display-value", True)
        self.display_size = options.get("display-size", True)
        self.human_size = options.get("human-size", True)
        self.use_absolute_address = options.get("absolute-address", False)
        self.hex_address = options.get("hex-address", False)
        self.indent = "   "

    def writeLine(self, text):
        raise NotImplementedError()

    def display(self):
        try:
            self._display(self.field_set)
        except IOError, err:
            if err[0] == errno.EPIPE:
                sys.exit(0)
            else:
                raise

    def formatAddress(self, field):
        if self.use_absolute_address:
            address = field.absolute_address
        else:
            address = field.address
        display_bits = address % 8 or field.size % 8
        if self.hex_address:
            if display_bits:
                return "%04x.%x" % (address/8, address%8)
            else:
                return "%04x" % (address/8)
        else:
            if display_bits:
                return "%u.%u" % (address/8, address%8)
            else:
                return "%u" % (address/8)

    def formatSize(self, field):
        if field.size % 8:
            return "%s bits" % field.size
        elif not self.human_size:
            return "%s bytes" % (field.size / 8)
        else:
            return humanFilesize(field.size / 8)

    def displayField(self, indent, field):
        text = [indent]
        if field.size:
            text.append("%s) %s" % (self.formatAddress(field), field.name))
        else:
            text.append("-> %s" % field.name)
        if self.display_value and field.hasValue():
            assert isinstance(field.display, (str, unicode))
            text.append("= %s" % field.display)
        if field.description:
            text.append(": %s" % field.description)
        if field.size and self.display_size:
            text.append(" (size %s)" % self.formatSize(field))
        self.writeLine("".join(text))

    def displayFieldSet(self, field_set):
        if field_set.description:
            text = "%s (%s)" % (field_set.name, field_set.description)
        else:
            text = field_set.name
        self.writeLine(text)

    def _display(self, field_set, indent="", depth=1):
        if depth == 1:
            self.displayFieldSet(field_set)
        if (self.max_depth is not None) and (self.max_depth < depth):
            self.writeLine("%s(...)" % indent)
            return
        for field in field_set:
            self.displayField(indent, field)
            if field.is_field_set:
                self._display(field, indent+self.indent, depth+1)

